<?php

namespace Drp\SimpleCms\Test\Feature;

use Drp\SimpleCms\Contracts\Publishable;
use Drp\SimpleCms\Test\Support\Fakes\TestContent;
use Drp\SimpleCms\Test\Support\Helpers\InteractsWithPreviewMode;
use Drp\SimpleCms\Test\Support\Helpers\InteractsWithUsers;
use Drp\SimpleCms\Test\TestCase;
use Illuminate\Support\Facades\Notification;

class AdminPreviewModeTest extends TestCase
{
    use InteractsWithUsers, InteractsWithPreviewMode;

    /** @test */
    public function only_publlished_content_is_shown_when_not_in_preview_mode()
    {
        factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create([
                'title' => 'Title',
                'content' => 'Original content',
            ]);

        $draftContent = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => 1,
                'title' => 'Title',
                'content' => 'Edited content',
            ]);

        $this
            ->actingAsAdmin()
            ->visit($draftContent->frontEndUrl())
            ->dontSeeText('Edited content')
            ->seeText('Original content');
    }

    /** @test */
    public function an_admin_can_enable_preview_mode()
    {
        $this->actingAsAdmin()
            ->visitRoute('admin.preview.enable')
            ->assertPreviewModeIsEnabled()
            ->seePageIs('/home')
            ->seeLink('Disable Preview Mode', route('admin.preview.disable'))
            ->seeInSession('previewModeRedirectUrl', route('admin.dashboard'));
    }

    /** @test */
    public function an_admin_can_disable_preview_mode()
    {
        $this->enablePreviewMode('')
            ->actingAsAdmin()
            ->visitRoute('admin.preview.disable')
            ->assertPreviewModeIsDisabled()
            ->assertSessionMissing('previewMode');
    }

    /** @test */
    public function an_admin_is_redirected_to_previous_url_when_disable_preview_mode()
    {
        $this->enablePreviewMode(route('admin.content.index', 'test-content'))
            ->actingAsAdmin()
            ->visitRoute('admin.preview.disable')
            ->assertPreviewModeIsDisabled()
            ->seeRouteIs('admin.content.index', 'test-content')
            ->assertSessionMissing('previewModeRedirectUrl');
    }

    /** @test */
    public function an_admin_is_redirected_to_admin_if_the_link_came_from_outside_the_site()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        session(['_previous.url' => 'http://www.drpgroup.com']);

        $this->enablePreviewMode(route('admin.content.index', 'test-content'))
            ->actingAsAdmin()
            ->visitRoute('admin.preview.enable', [
                'previewableType' => $content->contentKey(),
                'previewableId' => $content->id,
            ])
            ->seeInSession('previewModeRedirectUrl', route('admin.dashboard'));
    }


    /** @test */
    public function an_admin_can_see_draft_content_when_in_preview_mode()
    {
        factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create([
                'title' => 'Title',
                'content' => 'Original content',
            ]);

        $draftContent = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => 1,
                'title' => 'Title',
                'content' => 'Edited content',
            ]);

        $this->enablePreviewMode()
            ->actingAsAdmin()
            ->visit($draftContent->frontEndUrl())
            ->seeText('Draft Version')
            ->seeText('Edited content')
            ->dontSeeText('Original content');

    }

    /** @test */
    public function an_admin_can_see_draft_content_when_in_preview_mode_through_content_specific_link()
    {
        factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create([
                'title' => 'Title',
                'content' => 'Original content',
            ]);

        $draftContent = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => 1,
                'title' => 'Title',
                'content' => 'Edited content',
            ]);

        $this->actingAsAdmin()
            ->visitRoute('admin.content.manage', $draftContent->getContentRouteParameters())
            ->seeLink('Preview content', route('admin.preview.enable', [
                'previewableType' => $draftContent->contentKey(),
                'previewableId' => $draftContent->id,
            ]))
            ->click('Preview content')
            ->seePageIs($draftContent->frontEndUrl())
            ->seeText('Draft Version')
            ->seeText('Edited content')
            ->dontSeeText('Original content');
    }

    /** @test */
    public function an_admin_can_view_preview_toolbar_when_preview_mode_is_enabled()
    {

        $this->enablePreviewMode()
            ->actingAsAdmin()
            ->visitRoute('home')
            ->assertPreviewModeIsEnabled()
            ->within('#preview-bar', function () {
                $this->seeLink('Disable Preview Mode', route('admin.preview.disable'));
            });
    }

    /** @test */
    public function an_admin_is_shown_the_edit_link()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $this->enablePreviewMode()
            ->actingAsAdmin()
            ->visit($content->frontEndUrl())
            ->seeLink('Edit', $content->adminEditUrl());
    }

    /** @test */
    public function an_admin_previewer_is_not_shown_the_edit_link()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $this->enablePreviewMode()
            ->actingAsAdminPreviewer()
            ->visit($content->frontEndUrl())
            ->dontSeeLink('Edit', $content->adminEditUrl());
    }

    /** @test */
    public function when_a_direct_preview_is_access_a_notification_is_shown_if_this_has_been_published()
    {

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create([
                'title' => 'Original title'
            ]);

        $this->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->dontSeeLink('Preview Study', route('admin.preview.enable', [
                'previewableType' => $content->contentKey(),
                'previewableId' => $content->id,
            ]))
            ->visitRoute('admin.preview.enable', [
                'previewableType' => $content->contentKey(),
                'previewableId' => $content->id,
            ])
            ->seePageIs($content->frontEndUrl())
            ->seeText('Published Version')
            ->seeText('Original title')
            ->seeText(trans('simpleCms::preview.already_published'));

    }

    /** @test */
    public function an_admin_can_select_a_user_to_send_a_preview_link_to()
    {

        $this->markTestSkipped();

        $this->disableExceptionHandler();

        Notification::fake();

        $publishedContent = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => 1,
                'nameplate_id' => $this->nameplate->id,
            ]);

        $user = factory(User::class)
            ->states(User::ROLE_ADMIN_PREVIEWER)
            ->create();

        $this->actingAsAdmin()
            ->visit($content->frontEndUrl())
            ->select($user->id, 'user_id')
            ->submitForm('Send Preview')
            ->seeInDatabase('previews', [
                'user_id' => $user->id,
                'previewable_id' => $content->id,
                'previewable_type' => 'vehicle-themes',
            ]);

        Notification::assertSentTo(
            $user,
            PreviewSent::class,
            function ($notification, $channels) {
                return $notification->preview->id == 1;
            }
        );
    }

    /** @test */
    public function when_a_user_views_study_that_has_become_published_they_are_told_so()
    {
        $this->markTestSkipped();

        $publishedContent = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => 1,
                'nameplate_id' => $this->nameplate->id,
            ]);

        $user = factory(User::class)
            ->states(User::ROLE_ADMIN_PREVIEWER)
            ->create();

        $this->actingAsAdmin()
            ->visit($content->frontEndUrl())
            ->select($user->id, 'user_id')
            ->submitForm('Send Preview')
            ->seeInDatabase('previews', [
                'user_id' => $user->id,
                'previewable_id' => $content->id,
                'previewable_type' => 'vehicle-themes',
            ]);

        $content->updatePublishStatus();

        $this->seeIsSoftDeletedInDatabase('previews', [
            'user_id' => $user->id,
            'previewable_id' => $content->id,
            'previewable_type' => 'vehicle-themes',
        ]);

        $this->visitRoute('admin.preview.enable', [
            'previewableType' => $content->key,
            'previewableId' => $content->id,
        ])
            ->seeText('The preview link has expired, the current Study is now published.');
    }

    /** @test */
    public function previews_entries_for_content_that_have_been_published_are_removed()
    {
        $this->markTestSkipped();

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create();

        $preview = factory(Preview::class)
            ->create([
                'previewable_id' => $content->id,
                'previewable_type' => 'markets',
            ]);

        $this->seeInDatabase('previews', [
            'user_id' => $preview->user_id,
            'previewable_id' => $content->id,
            'previewable_type' => 'markets',
        ]);

        $content->updatePublishStatus();

        $this->seeIsSoftDeletedInDatabase('previews', [
            'user_id' => $preview->user_id,
            'previewable_id' => $content->id,
            'previewable_type' => 'markets',
        ]);
    }


    /** @test */
    public function an_admin_can_view_edit_links()
    {

        $this->markTestSkipped();

        $content = $this->createTestContentWithNameplate(Publishable::STATUS_PUBLISHED, [
            'markets' => 'Content TestContent',
        ]);

        $market = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'nameplate_id' => $content->nameplate_id,
            ]);

        $this->actingAsAdmin()
            ->enablePreviewMode()
            ->visitRoute('target-customer-content.markets.index', $market->frontEndRouteParameters())
            ->seePublishedEditLink($content)
            ->seeDraftOrInReviewEditLink($market);
    }
}
